"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fr = exports.deg_rad = exports.deg_grad = exports.deg_turn = exports.deg = exports.cm_pc = exports.cm_pt = exports.cm_mm = exports.cm_in = exports.cm = exports.ms_sec = exports.ms = exports.vmax_f = exports.vmax = exports.vmin_f = exports.vmin = exports.vh_f = exports.vh = exports.vw_f = exports.vw = exports.pct_f = exports.pct = exports.ex = exports.ch = exports.em = exports.rem = exports.px = exports.unitVal = exports.UnitValue = void 0;
exports.unitOf = unitOf;
/**
 * Abstract class that holds a value and its unit.
 * Provides a `toString` and `valueOf` methods, and nothing else.
 *
 * Use the `unitVal` helper to create nicely typed instances of this class.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
/*#__NO_SIDE_EFFECTS__*/
class UnitValue {
    constructor(unit, value) {
        // TODO: Decide if we want to error/warn on empoty `unit` strings
        // TODO: Decide if we want to error/warn on invalid UnitValue types
        this.value = typeof value === 'number' ? value : value.value;
        this.unit = unit;
    }
    toString() {
        return this.value + this.unit;
    }
    valueOf() {
        return this.value;
    }
    /** @deprecated   DO NOT USE.
     * — This is a typing hack to allow direct printing into
     * CSS templates with the popular "styled-components" library.
     * This method is an alias of `.toString()` */
    getName() {
        return this.toString();
    }
}
exports.UnitValue = UnitValue;
/**
 * Creates a custom `UnitValue` instance.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
/*#__NO_SIDE_EFFECTS__*/
const unitVal = (value, unit) => new UnitValue(unit, value);
exports.unitVal = unitVal;
/*#__NO_SIDE_EFFECTS__*/
function unitOf(item) {
    return item instanceof UnitValue ? item.unit : undefined;
}
/** Returns a `px` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const px = (n) => (0, exports.unitVal)(n, 'px');
exports.px = px;
/** Returns a `rem` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const rem = (n) => (0, exports.unitVal)(n, 'rem');
exports.rem = rem;
/** Returns a `em` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const em = (n) => (0, exports.unitVal)(n, 'em');
exports.em = em;
/** Returns a `ch` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const ch = (n) => (0, exports.unitVal)(n, 'ch');
exports.ch = ch;
/** Returns a `ex` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const ex = (n) => (0, exports.unitVal)(n, 'ex');
exports.ex = ex;
/** Returns a `pct` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const pct = (n) => (0, exports.unitVal)(n, '%');
exports.pct = pct;
/**
 * Convert fraction/proprtion to `%`.
 *
 * Multiplies the input by 100 before returning a `%` value (`PctValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const pct_f = (n) => (0, exports.pct)(n * 100);
exports.pct_f = pct_f;
/** Returns a `vw` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const vw = (n) => (0, exports.unitVal)(n, 'vw');
exports.vw = vw;
/**
 * Convert fraction/proprtion to `vw` per-centage.
 *
 * Multiplies the input by 100 before returning a `vw` value (`VwValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const vw_f = (n) => (0, exports.vw)(n * 100);
exports.vw_f = vw_f;
/** Returns a `vh` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const vh = (n) => (0, exports.unitVal)(n, 'vh');
exports.vh = vh;
/**
 * Convert fraction/proprtion to `vh` per-centage.
 *
 * Multiplies the input by 100 before returning a `vh` value (`VhValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const vh_f = (n) => (0, exports.vh)(n * 100);
exports.vh_f = vh_f;
/** Returns a `vmin` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const vmin = (n) => (0, exports.unitVal)(n, 'vmin');
exports.vmin = vmin;
/**
 * Convert fraction/proprtion to `vmin` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmin` value (`VminValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const vmin_f = (n) => (0, exports.vmin)(n * 100);
exports.vmin_f = vmin_f;
/** Returns a `vmax` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const vmax = (n) => (0, exports.unitVal)(n, 'vmax');
exports.vmax = vmax;
/**
 * Convert fraction/proprtion to `vmax` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmax` value (`VmaxValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const vmax_f = (n) => (0, exports.vmax)(n * 100);
exports.vmax_f = vmax_f;
/** Returns a `ms` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const ms = (n) => (0, exports.unitVal)(n, 'ms');
exports.ms = ms;
/**
 * Convert seconds (`s`) to `ms`.
 *
 * Multiplies the input by 1000 before returning a `ms` value (`MsValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const ms_sec = (n) => (0, exports.ms)(n * 1000);
exports.ms_sec = ms_sec;
/** Returns a `cm` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const cm = (n) => (0, exports.unitVal)(n, 'cm');
exports.cm = cm;
/**
 * Convert inches (`in`) to `cm`.
 *
 * Multiplies the input by 2.54 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const cm_in = (n) => (0, exports.cm)(n * 2.54);
exports.cm_in = cm_in;
/**
 * Convert millimeters (`mm`) to `cm`.
 *
 * Multiplies the input by .1 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const cm_mm = (n) => (0, exports.cm)(n * 0.1);
exports.cm_mm = cm_mm;
/**
 * Convert points (`pt`) to `cm`.
 *
 * Multiplies the input by 0.0352777778 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const cm_pt = (n) => (0, exports.cm)(n * 0.0352777778);
exports.cm_pt = cm_pt;
/**
 * Convert picas (`pc`) to `cm`.
 *
 * Multiplies the input by .42333333333 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const cm_pc = (n) => (0, exports.cm)(n * 0.42333333333);
exports.cm_pc = cm_pc;
/** Returns a `deg` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const deg = (n) => (0, exports.unitVal)(n, 'deg');
exports.deg = deg;
/**
 * Convert turns (`turn`) to `deg`.
 *
 * Multiplies the input by 360 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const deg_turn = (n) => (0, exports.deg)(n * 360);
exports.deg_turn = deg_turn;
const GRAD_TO_DEG = 360 / 400;
/**
 * Convert gradians (`grad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const deg_grad = (n) => (0, exports.deg)(n * GRAD_TO_DEG);
exports.deg_grad = deg_grad;
const RAD_TO_DEG = 360 / (2 * Math.PI);
/**
 * Convert radians (`rad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
const deg_rad = (n) => (0, exports.deg)(n * RAD_TO_DEG);
exports.deg_rad = deg_rad;
/** Returns a `fr` `UnitNumber` for grid sizing
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
const fr = (n) => (0, exports.unitVal)(n, 'fr');
exports.fr = fr;
