"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compileCSSFromJS = exports.compileCSS = void 0;
exports.compileCSSString = compileCSSString;
const autoprefixer_1 = __importDefault(require("autoprefixer"));
const cssnano_1 = __importDefault(require("cssnano"));
const promises_1 = require("fs/promises");
const path_1 = require("path");
const postcss_1 = __importDefault(require("postcss"));
const postcss_nested_1 = __importDefault(require("postcss-nested"));
const postcss_scss_1 = __importDefault(require("postcss-scss"));
const getExportedCSS_js_1 = require("./_compiler/getExportedCSS.js");
const prettifyCSS_js_1 = require("./_compiler/prettifyCSS.js");
const resolveOutputFiles_js_1 = require("./_compiler/resolveOutputFiles.js");
const makeFile = (outFile, contents) => {
    const targetDir = (0, path_1.dirname)(outFile);
    return (0, promises_1.access)(targetDir)
        .catch(() => (0, promises_1.mkdir)(targetDir, { recursive: true }))
        .then(() => (0, promises_1.writeFile)(outFile, contents));
};
const makeProcessCSS = (options) => {
    const { nested = true, minify, prettify } = options;
    const postcssPlugins = [];
    if (nested) {
        const nestedOpts = nested !== true ? nested : {};
        postcssPlugins.push((0, postcss_nested_1.default)(nestedOpts));
    }
    postcssPlugins.push(autoprefixer_1.default);
    if (minify) {
        postcssPlugins.push((0, cssnano_1.default)({ preset: 'default' }));
    }
    const postPostss = !minify && prettify
        ? (0, prettifyCSS_js_1.makePrettifyCSS)(prettify, options.outdir)
        : (css) => css;
    return (css) => {
        // NOTE: Functions/mixins commonly introduce stray ";" after curly-brace blocks.
        // Those semicolons then get parsed as weird/invalid selector tokens.
        css = css.replace(/\}(\s*;)+/g, '}');
        return (0, postcss_1.default)(postcssPlugins)
            .process(css, {
            from: undefined,
            // Converts inline comments to comment blocks
            parser: postcss_scss_1.default,
        })
            .then((result) => postPostss(result.css));
    };
};
const bannerify = (css, options) => {
    if (options.banner) {
        css = `${options.banner}\n${css}`;
    }
    if (options.footer) {
        css = `${css.replace(/\n$/, '')}\n${options.footer}\n`;
    }
    return css;
};
/**
 * Works in pretty much the same way as the CLI.
 *
 * Takes a list of files to read, and returns an Array of result objects each
 * containing the compiled CSS and the resolved output file path.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#compilecss-from-files
 */
const compileCSS = (sourceFiles, options = {}) => {
    const processCSS = makeProcessCSS(options);
    return Promise.all((0, resolveOutputFiles_js_1.resolveOutputFiles)(sourceFiles, options).map((args) => (0, getExportedCSS_js_1.getExportedCSS)(args.inFile)
        .then(processCSS)
        .then(async (css) => {
        css = bannerify(css, options);
        if (options.write !== false) {
            await makeFile(args.outFile, css);
            return Object.assign({}, args);
        }
        return Object.assign({ css }, args);
    })
        .catch((e) => {
        const message = e instanceof Error ? e.message : String(e);
        throw new Error(`Processing ${args.inFile}\n  ${message.replace(/\n/g, '\n  ')}`);
    })));
};
exports.compileCSS = compileCSS;
// ---------------------------------------------------------------------------
/**
 * Compiles CSS from a JavaScript source string. This may be the preferable
 * method when working with bundlers such as `esbuild`.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#compilecssfromjs
 */
const compileCSSFromJS = (scriptStrings, options) => {
    const _tempfiles = {};
    const unlinkTempFile = (fileName) => {
        (0, promises_1.unlink)(fileName).catch(() => undefined);
        delete _tempfiles[fileName];
    };
    const emergencyUnlink = () => Object.keys(_tempfiles).forEach(unlinkTempFile);
    process.on('exit', emergencyUnlink);
    const compilation = Promise.all(scriptStrings.map(async ({ fileName, content }) => {
        _tempfiles[fileName] = true;
        await makeFile(fileName, content);
        return fileName;
    }))
        .then((inputFiles) => (0, exports.compileCSS)(inputFiles, options).catch((err) => {
        inputFiles.forEach(unlinkTempFile);
        throw err;
    }))
        .then((rets) => {
        rets.map((ret) => ret.inFile).forEach(unlinkTempFile);
        process.off('exit', emergencyUnlink);
        return rets;
    });
    return compilation;
};
exports.compileCSSFromJS = compileCSSFromJS;
function compileCSSString(css, options = {}) {
    const processCSS = makeProcessCSS(options);
    const compileString = (css) => processCSS(css).then((css) => bannerify(css, options));
    if (typeof css === 'string') {
        return compileString(css);
    }
    return Promise.all(css.map(compileString));
}
