/** Filters out all falsy values except `0`. */
const notFalsy = (val) => !!val || val === 0;
/**
 * Dumb(-ish) tagged template literal that returns a `string`. It primarily
 * guarantees nice syntax highlighting and code-completion in VSCode by using a
 * well-known name.
 *
 * It also provides a few convenience features.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#css-templater
 */
/*#__NO_SIDE_EFFECTS__*/
export const css = function (strings, ...values) {
    const valCount = values.length;
    return strings
        .map((str, i) => {
        if (i === valCount) {
            return str;
        }
        const rawValue = values[i];
        const value = Array.isArray(rawValue)
            ? rawValue.filter(notFalsy).join(' ')
            : typeof rawValue === 'function'
                ? rawValue()
                : rawValue;
        return str + String(notFalsy(value) ? value : '');
    })
        .join('');
};
// ---------------------------------------------------------------------------
/**
 * An alias for the ` css`` ` templater function, for cases where you're writing a
 * standalone CSS value or other out-of-context CSS snippets, and you wish to
 * disable VSCode's syntax highlighting and error checking.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#cssval-templater
 */
/*#__NO_SIDE_EFFECTS__*/
export const cssVal = css;
/*#__NO_SIDE_EFFECTS__*/
export function media(query, cssContent) {
    const mediaWrappinator = (cssContent) => css `
      ${'@at-root (without: media)'} {
        @media ${query} {
          ${cssContent}
        }
      }
    `;
    if (arguments.length === 1) {
        // Curry!
        return mediaWrappinator;
    }
    return mediaWrappinator(cssContent || '');
}
/**
 * Helper to convert a value to a "quoted string".
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#str-quoted-string-printer
 */
/*#__NO_SIDE_EFFECTS__*/
export const str = (string) => JSON.stringify(`${string}`);
