import { ColorValue } from './colors.js';
import { CssString } from './css.js';
import { UnitValue } from './units.js';
declare const _CssVarString__Brand: unique symbol;
/** CSS `var(--custom-property, fallback)` string  */
export type CssVarString = CssString & {
    [_CssVarString__Brand]: true;
};
export type VariableValue = string | number | UnitValue | ColorValue | VariablePrinter;
declare const IS_PRINTER: unique symbol;
type VarsMap<T extends string> = Record<T, VariablePrinter>;
export type VariableStyles<T extends string> = {
    /**
     * Holds a readonly `Record<T, VariablePrinter>` object where the
     * `VariablePrinter`s emit the CSS variable names wrapped in `var()`, ready to be
     * used as CSS values … with the option of passing a default/fallback value via
     * the `.or() method`.
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variablestylesvars
     */
    readonly vars: VarsMap<T>;
    /**
     * Lets you type-safely write values for **all** the defined CSS variables into a
     * CSS rule block. Property names not matching `T` are dropped/ignored.
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variablestylesdeclare
     */
    declare(vars: Record<T, VariableValue>): CssString;
    /**
     * Similar to the `.declare()` method, but can be used to re-declare (i.e.
     * override) only some of of the CSS variables `T`. Again, property names not
     * matching `T` are ignored/dropped.
     *
     * Furthermore, values of `null`, `undefined`, `false` are interpreted as
     * "missing", and the property is ignored/dropped.
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variablestylesoverride
     */
    override(vars: Partial<Record<T, VariableValue | false | null>>): CssString;
};
export type VariablePrinter = Readonly<{
    /** Prints the CSS varible string with a fallback/default value paramter. */
    or(defaultValue: VariableValue): CssVarString;
    readonly cssName: string;
    toString(): string;
    toJSON(): string;
    /** @deprecated Typing hack to allow direct printing into styled-components CSS templates. This method is an alias of `.toString()` */
    getName(): string;
}> & {
    [IS_PRINTER]: true;
};
export type VariableOptions = {
    /**
     * Custom name validation RegExp, for if you want/need to allow names
     * more complex than the default setting allows.
     *
     * Default: `/^[a-z0-9_-]+$/i`
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variableoptions
     */
    nameRe?: RegExp;
    /**
     * Maps weird/wonky JavaScript property names to CSS-friendly
     * css custom property names.
     *
     * Default: `(name) => name`
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variableoptions
     */
    toCSSName?: (name: string) => string;
    /**
     * Prefix that gets added to all CSS printed variable names.
     *
     * The namespace is neither validated nor transformed in any way, except that
     * spaces and other invalid characters are silently stripped away.
     *
     * @see https://github.com/maranomynet/es-in-css/tree/v0.7#variableoptions
     */
    namespace?: string;
};
/**
 * Helper to provide type-safety and code-completion when using CSS custom
 * properties (CSS variables) at scale.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#makevariables-helper
 */
export declare const makeVariables: {
    <T extends string>(variableTokens: Array<T>, options?: VariableOptions): VariableStyles<T>;
    isVar(value: unknown): value is VariablePrinter;
    join<VArr extends Array<VariableStyles<string>>>(...varDatas: VArr): VariableStyles<VArr extends Array<infer V> ? (V extends VariableStyles<infer T> ? T : never) : never>;
};
declare const _RawCssVarString__Brand: unique symbol;
/** @deprecated  Use `CssString` type instead (Will be removed in v0.8) */
export type RawCssVarString = string & {
    [_RawCssVarString__Brand]?: true;
};
export {};
