const IS_PRINTER = Symbol();
const DEFAULT_NAME_RE = /^[a-z0-9_-]+$/i;
const DEFAULT_NAME_MAPPER = (name) => name;
// ---------------------------------------------------------------------------
const makeVariablePrinter = (name) => {
    const cssName = `--${name}`;
    const varString = `var(${cssName})`;
    const toString = () => varString;
    return {
        cssName,
        or: (defaultValue) => defaultValue
            ? varString.replace(/\)$/, `, ${defaultValue})`)
            : varString,
        [IS_PRINTER]: true,
        toString,
        toJSON: toString,
        getName: toString,
    };
};
// ---------------------------------------------------------------------------
const makeDeclarations = (vars, allowed) => {
    return Object.entries(vars)
        .map(([key, value]) => {
        const printer = allowed[key];
        if (!printer || value == null || value === false) {
            return '';
        }
        return `${printer.cssName}: ${String(value).trim()};\n`;
    })
        .join('');
};
// ===========================================================================
const assertValidNameRe = (nameRe) => {
    if (nameRe && !/^\/\^.+\$\/[igm]*$/.test(String(nameRe))) {
        throw new Error('Custom variable name RegExp must check the whole name (i.e. start with a `^` and end with a `$`)');
    }
};
const assertValidName = (name, nameRe = DEFAULT_NAME_RE) => {
    if (!name || !nameRe.test(name)) {
        throw new Error(`Only CSS variable names matching ${nameRe} are allowed.\nDisallowed name: ${name}`);
    }
};
/**
 * Helper to provide type-safety and code-completion when using CSS custom
 * properties (CSS variables) at scale.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#makevariables-helper
 */
/*#__NO_SIDE_EFFECTS__*/
export const makeVariables = (variableTokens, options = {}) => {
    assertValidNameRe(options.nameRe);
    const { nameRe, toCSSName = DEFAULT_NAME_MAPPER } = options;
    const namespace = (options.namespace || '').replace(/[\s{};@():[\]]/g, '');
    const vars = Object.fromEntries(variableTokens.map((name) => {
        assertValidName(name, nameRe);
        return [name, makeVariablePrinter(namespace + toCSSName(name))];
    }));
    return {
        vars,
        declare: (declarations) => makeDeclarations(declarations, vars),
        override: (overrides) => makeDeclarations(overrides, vars),
    };
};
// ---------------------------------------------------------------------------
/**
 * A helper that checks if an input value is of type `VariablePrinter`.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#makevariablesisvar-helper
 */
makeVariables.isVar = (value) => typeof value === 'function' && IS_PRINTER in value;
// ---------------------------------------------------------------------------
/**
 * This helper combines the variable values and declaration methods from multiple
 * `VariableStyles` objects into a new, larger `VariableStyles` object.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#makevariablesjoin-composition-helper
 */
makeVariables.join = (...varDatas) => {
    const vars = Object.assign({}, ...varDatas.map((d) => d.vars));
    return {
        vars,
        declare: (declarations) => makeDeclarations(declarations, vars),
        override: (overrides) => makeDeclarations(overrides, vars),
    };
};
