/**
 * Abstract class that holds a value and its unit.
 * Provides a `toString` and `valueOf` methods, and nothing else.
 *
 * Use the `unitVal` helper to create nicely typed instances of this class.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
export declare class UnitValue<U extends string = string> {
    constructor(unit: U, value: PlainNumber | UnitValue<U>);
    readonly value: number;
    readonly unit: U;
    toString(): string;
    valueOf(): number;
    /** @deprecated   DO NOT USE.
     * — This is a typing hack to allow direct printing into
     * CSS templates with the popular "styled-components" library.
     * This method is an alias of `.toString()` */
    getName(): string;
}
/**
 * A UnitValue instance that can be casually used in plain calculations. \
 * However, It will fail `typeof x === "number"` checks.
 */
type UnitNumber<U extends string> = number & UnitValue<U>;
/**
 * Creates a custom `UnitValue` instance.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
export declare const unitVal: <U extends string>(value: PlainNumber | UnitValue<U>, unit: U) => UnitNumber<U>;
/**
 * Checks if its given argument is a `UnitValue` instance and returns its unit
 * string. Otherwise returns `undefined`.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitof-helper
 */
export declare function unitOf(item: PlainNumber): undefined;
export declare function unitOf<U extends string>(item: UnitValue<U>): U;
export declare function unitOf<U extends string>(item: number | UnitValue<U>): U | undefined;
/**
 * A "vanilla" `number` value that is NOT a `UnitValue`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-types
 */
export type PlainNumber = number & {
    unit?: never;
};
export type PxValue = UnitNumber<'px'>;
/** Returns a `px` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const px: (n: PlainNumber | PxValue) => PxValue;
export type RemValue = UnitNumber<'rem'>;
/** Returns a `rem` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const rem: (n: PlainNumber | RemValue) => RemValue;
export type EmValue = UnitNumber<'em'>;
/** Returns a `em` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const em: (n: PlainNumber | EmValue) => EmValue;
export type ChValue = UnitNumber<'ch'>;
/** Returns a `ch` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const ch: (n: PlainNumber | ChValue) => ChValue;
export type ExValue = UnitNumber<'ex'>;
/** Returns a `ex` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const ex: (n: PlainNumber | ExValue) => ExValue;
export type PctValue = UnitNumber<'%'>;
/** Returns a `pct` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const pct: (n: PlainNumber | PctValue) => PctValue;
/**
 * Convert fraction/proprtion to `%`.
 *
 * Multiplies the input by 100 before returning a `%` value (`PctValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const pct_f: (n: PlainNumber) => PctValue;
export type VwValue = UnitNumber<'vw'>;
/** Returns a `vw` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const vw: (n: PlainNumber | VwValue) => VwValue;
/**
 * Convert fraction/proprtion to `vw` per-centage.
 *
 * Multiplies the input by 100 before returning a `vw` value (`VwValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const vw_f: (n: PlainNumber) => VwValue;
export type VhValue = UnitNumber<'vh'>;
/** Returns a `vh` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const vh: (n: PlainNumber | VhValue) => VhValue;
/**
 * Convert fraction/proprtion to `vh` per-centage.
 *
 * Multiplies the input by 100 before returning a `vh` value (`VhValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const vh_f: (n: PlainNumber) => VhValue;
export type VminValue = UnitNumber<'vmin'>;
/** Returns a `vmin` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const vmin: (n: PlainNumber | VminValue) => VminValue;
/**
 * Convert fraction/proprtion to `vmin` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmin` value (`VminValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const vmin_f: (n: PlainNumber) => VminValue;
export type VmaxValue = UnitNumber<'vmax'>;
/** Returns a `vmax` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const vmax: (n: PlainNumber | VmaxValue) => VmaxValue;
/**
 * Convert fraction/proprtion to `vmax` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmax` value (`VmaxValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const vmax_f: (n: PlainNumber) => VmaxValue;
export type MsValue = UnitNumber<'ms'>;
/** Returns a `ms` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const ms: (n: PlainNumber | MsValue) => MsValue;
/**
 * Convert seconds (`s`) to `ms`.
 *
 * Multiplies the input by 1000 before returning a `ms` value (`MsValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const ms_sec: (n: PlainNumber) => MsValue;
export type CmValue = UnitNumber<'cm'>;
/** Returns a `cm` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const cm: (n: PlainNumber | CmValue) => CmValue;
/**
 * Convert inches (`in`) to `cm`.
 *
 * Multiplies the input by 2.54 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const cm_in: (n: PlainNumber) => CmValue;
/**
 * Convert millimeters (`mm`) to `cm`.
 *
 * Multiplies the input by .1 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const cm_mm: (n: PlainNumber) => CmValue;
/**
 * Convert points (`pt`) to `cm`.
 *
 * Multiplies the input by 0.0352777778 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const cm_pt: (n: PlainNumber) => CmValue;
/**
 * Convert picas (`pc`) to `cm`.
 *
 * Multiplies the input by .42333333333 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const cm_pc: (n: PlainNumber) => CmValue;
export type DegValue = UnitNumber<'deg'>;
/** Returns a `deg` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const deg: (n: PlainNumber | DegValue) => DegValue;
/**
 * Convert turns (`turn`) to `deg`.
 *
 * Multiplies the input by 360 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const deg_turn: (n: PlainNumber) => DegValue;
/**
 * Convert gradians (`grad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const deg_grad: (n: PlainNumber) => DegValue;
/**
 * Convert radians (`rad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
export declare const deg_rad: (n: PlainNumber) => DegValue;
export type FrValue = UnitNumber<'fr'>;
/** Returns a `fr` `UnitNumber` for grid sizing
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
export declare const fr: (n: PlainNumber | FrValue) => FrValue;
/**
 * All container proportional units: `%`, `vw`, etc.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-types
 */
export type LayoutRelativeValue = PctValue | VwValue | VhValue | VminValue | VmaxValue;
/**
 * All text proportional units: `em`, `rem`, etc.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-types
 */
export type FontRelativeValue = EmValue | RemValue | ChValue | ExValue;
/**
 * All fixed/physical units (`px`, `cm`), plus container-proportional and text-proportional units
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-types
 */
export type LengthValue = PxValue | CmValue | LayoutRelativeValue | FontRelativeValue;
export {};
