/**
 * Abstract class that holds a value and its unit.
 * Provides a `toString` and `valueOf` methods, and nothing else.
 *
 * Use the `unitVal` helper to create nicely typed instances of this class.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
/*#__NO_SIDE_EFFECTS__*/
export class UnitValue {
    constructor(unit, value) {
        // TODO: Decide if we want to error/warn on empoty `unit` strings
        // TODO: Decide if we want to error/warn on invalid UnitValue types
        this.value = typeof value === 'number' ? value : value.value;
        this.unit = unit;
    }
    toString() {
        return this.value + this.unit;
    }
    valueOf() {
        return this.value;
    }
    /** @deprecated   DO NOT USE.
     * — This is a typing hack to allow direct printing into
     * CSS templates with the popular "styled-components" library.
     * This method is an alias of `.toString()` */
    getName() {
        return this.toString();
    }
}
/**
 * Creates a custom `UnitValue` instance.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unitval-helper
 */
/*#__NO_SIDE_EFFECTS__*/
export const unitVal = (value, unit) => new UnitValue(unit, value);
/*#__NO_SIDE_EFFECTS__*/
export function unitOf(item) {
    return item instanceof UnitValue ? item.unit : undefined;
}
/** Returns a `px` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const px = (n) => unitVal(n, 'px');
/** Returns a `rem` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const rem = (n) => unitVal(n, 'rem');
/** Returns a `em` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const em = (n) => unitVal(n, 'em');
/** Returns a `ch` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const ch = (n) => unitVal(n, 'ch');
/** Returns a `ex` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const ex = (n) => unitVal(n, 'ex');
/** Returns a `pct` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const pct = (n) => unitVal(n, '%');
/**
 * Convert fraction/proprtion to `%`.
 *
 * Multiplies the input by 100 before returning a `%` value (`PctValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const pct_f = (n) => pct(n * 100);
/** Returns a `vw` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const vw = (n) => unitVal(n, 'vw');
/**
 * Convert fraction/proprtion to `vw` per-centage.
 *
 * Multiplies the input by 100 before returning a `vw` value (`VwValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const vw_f = (n) => vw(n * 100);
/** Returns a `vh` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const vh = (n) => unitVal(n, 'vh');
/**
 * Convert fraction/proprtion to `vh` per-centage.
 *
 * Multiplies the input by 100 before returning a `vh` value (`VhValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const vh_f = (n) => vh(n * 100);
/** Returns a `vmin` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const vmin = (n) => unitVal(n, 'vmin');
/**
 * Convert fraction/proprtion to `vmin` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmin` value (`VminValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const vmin_f = (n) => vmin(n * 100);
/** Returns a `vmax` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const vmax = (n) => unitVal(n, 'vmax');
/**
 * Convert fraction/proprtion to `vmax` per-centage.
 *
 * Multiplies the input by 100 before returning a `vmax` value (`VmaxValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const vmax_f = (n) => vmax(n * 100);
/** Returns a `ms` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const ms = (n) => unitVal(n, 'ms');
/**
 * Convert seconds (`s`) to `ms`.
 *
 * Multiplies the input by 1000 before returning a `ms` value (`MsValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const ms_sec = (n) => ms(n * 1000);
/** Returns a `cm` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const cm = (n) => unitVal(n, 'cm');
/**
 * Convert inches (`in`) to `cm`.
 *
 * Multiplies the input by 2.54 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const cm_in = (n) => cm(n * 2.54);
/**
 * Convert millimeters (`mm`) to `cm`.
 *
 * Multiplies the input by .1 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const cm_mm = (n) => cm(n * 0.1);
/**
 * Convert points (`pt`) to `cm`.
 *
 * Multiplies the input by 0.0352777778 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const cm_pt = (n) => cm(n * 0.0352777778);
/**
 * Convert picas (`pc`) to `cm`.
 *
 * Multiplies the input by .42333333333 before returning a `cm` value (`CmValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const cm_pc = (n) => cm(n * 0.42333333333);
/** Returns a `deg` `UnitNumber`
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const deg = (n) => unitVal(n, 'deg');
/**
 * Convert turns (`turn`) to `deg`.
 *
 * Multiplies the input by 360 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const deg_turn = (n) => deg(n * 360);
const GRAD_TO_DEG = 360 / 400;
/**
 * Convert gradians (`grad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const deg_grad = (n) => deg(n * GRAD_TO_DEG);
const RAD_TO_DEG = 360 / (2 * Math.PI);
/**
 * Convert radians (`rad`) to `deg`.
 *
 * Multiplies the input by 0.9 before returning a `deg` value (`DegValue`)
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-converters
 */
/*#__NO_SIDE_EFFECTS__*/
export const deg_rad = (n) => deg(n * RAD_TO_DEG);
/** Returns a `fr` `UnitNumber` for grid sizing
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#unit-value-helpers
 */
/*#__NO_SIDE_EFFECTS__*/
export const fr = (n) => unitVal(n, 'fr');
