import autoprefixer from 'autoprefixer';
import cssnano from 'cssnano';
import { access, mkdir, unlink, writeFile } from 'fs/promises';
import { dirname } from 'path';
import postcss from 'postcss';
import nestedPlugin from 'postcss-nested';
import scss from 'postcss-scss';
import { getExportedCSS } from './_compiler/getExportedCSS.js';
import { makePrettifyCSS } from './_compiler/prettifyCSS.js';
import { resolveOutputFiles } from './_compiler/resolveOutputFiles.js';
const makeFile = (outFile, contents) => {
    const targetDir = dirname(outFile);
    return access(targetDir)
        .catch(() => mkdir(targetDir, { recursive: true }))
        .then(() => writeFile(outFile, contents));
};
const makeProcessCSS = (options) => {
    const { nested = true, minify, prettify } = options;
    const postcssPlugins = [];
    if (nested) {
        const nestedOpts = nested !== true ? nested : {};
        postcssPlugins.push(nestedPlugin(nestedOpts));
    }
    postcssPlugins.push(autoprefixer);
    if (minify) {
        postcssPlugins.push(cssnano({ preset: 'default' }));
    }
    const postPostss = !minify && prettify
        ? makePrettifyCSS(prettify, options.outdir)
        : (css) => css;
    return (css) => {
        // NOTE: Functions/mixins commonly introduce stray ";" after curly-brace blocks.
        // Those semicolons then get parsed as weird/invalid selector tokens.
        css = css.replace(/\}(\s*;)+/g, '}');
        return postcss(postcssPlugins)
            .process(css, {
            from: undefined,
            // Converts inline comments to comment blocks
            parser: scss,
        })
            .then((result) => postPostss(result.css));
    };
};
const bannerify = (css, options) => {
    if (options.banner) {
        css = `${options.banner}\n${css}`;
    }
    if (options.footer) {
        css = `${css.replace(/\n$/, '')}\n${options.footer}\n`;
    }
    return css;
};
/**
 * Works in pretty much the same way as the CLI.
 *
 * Takes a list of files to read, and returns an Array of result objects each
 * containing the compiled CSS and the resolved output file path.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#compilecss-from-files
 */
export const compileCSS = (sourceFiles, options = {}) => {
    const processCSS = makeProcessCSS(options);
    return Promise.all(resolveOutputFiles(sourceFiles, options).map((args) => getExportedCSS(args.inFile)
        .then(processCSS)
        .then(async (css) => {
        css = bannerify(css, options);
        if (options.write !== false) {
            await makeFile(args.outFile, css);
            return Object.assign({}, args);
        }
        return Object.assign({ css }, args);
    })
        .catch((e) => {
        const message = e instanceof Error ? e.message : String(e);
        throw new Error(`Processing ${args.inFile}\n  ${message.replace(/\n/g, '\n  ')}`);
    })));
};
// ---------------------------------------------------------------------------
/**
 * Compiles CSS from a JavaScript source string. This may be the preferable
 * method when working with bundlers such as `esbuild`.
 *
 * @see https://github.com/maranomynet/es-in-css/tree/v0.7#compilecssfromjs
 */
export const compileCSSFromJS = (scriptStrings, options) => {
    const _tempfiles = {};
    const unlinkTempFile = (fileName) => {
        unlink(fileName).catch(() => undefined);
        delete _tempfiles[fileName];
    };
    const emergencyUnlink = () => Object.keys(_tempfiles).forEach(unlinkTempFile);
    process.on('exit', emergencyUnlink);
    const compilation = Promise.all(scriptStrings.map(async ({ fileName, content }) => {
        _tempfiles[fileName] = true;
        await makeFile(fileName, content);
        return fileName;
    }))
        .then((inputFiles) => compileCSS(inputFiles, options).catch((err) => {
        inputFiles.forEach(unlinkTempFile);
        throw err;
    }))
        .then((rets) => {
        rets.map((ret) => ret.inFile).forEach(unlinkTempFile);
        process.off('exit', emergencyUnlink);
        return rets;
    });
    return compilation;
};
export function compileCSSString(css, options = {}) {
    const processCSS = makeProcessCSS(options);
    const compileString = (css) => processCSS(css).then((css) => bannerify(css, options));
    if (typeof css === 'string') {
        return compileString(css);
    }
    return Promise.all(css.map(compileString));
}
